/*-------------------------------------------------------------------*/
/* CBKM.C         Marko Tuononen                                     */
/*                                                                   */
/* Console interface for repeated K-means algorithm                  */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define ProgName       "CBKM"
#define VersionNumber  "Version 0.52" /* PF */
#define LastUpdated    "11.07.2009"
#define FACTFILE       "cbkm.fac"

/* ----------------------------------------------------------------- */

#include "parametr.c"
#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "random.h"
#include "reporting.h"
#include "kmeans.h"


/* ======================== PRINT ROUTINES =========================== */

void PrintInfo(void)
{
  PrintMessage("%s\t%s\t%s\n"
        "Repeated K-means algorithm.\n"
        "Usage: %s [%coption] <training set> [initial cb/pa] "
        "<result codebook>\n"
        "For example: %s bridge initial tmp\n\n  Options:\n",
        ProgName, VersionNumber, LastUpdated, ProgName, 
        OPTION_SYMBOL, ProgName);
  PrintOptions();
  PrintMessage("\n");
} 

/* ------------------------------------------------------------------ */

static char* PrintOperatingInfo(char* TSName, char* InName,
                                char* OutCBName, char* OutPAName,
                                int useInitial)
{ 
  char* str;
  str = KMeansInfo();

  if( Value(QuietLevel) >= 2 )
    {
    printf("\n%s %s %s\n", ProgName, VersionNumber, LastUpdated);
    PrintMessage("Training Set:     %s\n", TSName);
    PrintMessage("Codebook:         %s\n", OutCBName);
    if (useInitial)
      {
      PrintMessage("Initial %s",
                  ((useInitial == 1) ? "codebook:         "
                                     : "partition:        "));
      PrintMessage("%s\n", InName);
      }
    if (*OutPAName) 
      {
      PrintMessage("Result partition: %s\n", OutPAName);
      }
    printf("\n");
    PrintSelectedOptions();
    printf("\n");
    }
  return str;
}


/* ===========================  MAIN  ================================ */

int main(int argc, char* argv[])
{
  TRAININGSET   TS;
  CODEBOOK      CB;
  PARTITIONING  P;
  char          TSName[MAXFILENAME] = {'\0'};
  char          InName[MAXFILENAME] = {'\0'};
  char          OutCBName[MAXFILENAME] = {'\0'};
  char          OutPAName[MAXFILENAME] = {'\0'};
  char*         genMethod;
  int           useInitial = 0; 
  ParameterInfo paraminfo[3] = { { TSName,  FormatNameTS, 0, INFILE },
                                 { InName,  FormatNameCB, 1, INFILE },
                                 { OutCBName,  FormatNameCB, 0, OUTFILE } };
  
  initrandom(0);
  ParseParameters(argc, argv, 3, paraminfo);
  
  if (Value(SavePartition)) 
    {
    PickFileName(OutCBName, OutPAName);
    CheckFileName(OutPAName, FormatNamePA);
    }

  TS = CheckParameters(TSName, OutCBName, OutPAName, InName,
                       Value(Clusters), Value(OverWrite));
  
  useInitial = ReadInitialCBorPA(InName, Value(Clusters), &TS, &CB, &P);  

  genMethod = PrintOperatingInfo(TSName, InName, OutCBName, 
                                 OutPAName, useInitial);
  
  if (PerformKMeans(&TS, &CB, &P, Value(Clusters), Value(Repeats), 
		    Value(InitMethod), Value(QuietLevel), useInitial))
  {
    ErrorMessage("\nERROR: Clustering failed!\n\n");
    FreeCodebook(&TS);
    FreeCodebook(&CB);
    FreePartitioning(&P);
    free(genMethod);    
    ExitProcessing(FATAL_ERROR);
  }

  AddGenerationMethod(&CB, genMethod); 
  WriteCodebook(OutCBName, &CB, Value(OverWrite));
  
  if (Value(SavePartition))
  {
    WritePartitioning(OutPAName, &P, &TS, Value(OverWrite));
  }
  
  FreeCodebook(&TS);
  FreeCodebook(&CB);
  FreePartitioning(&P);
  free(genMethod);
 
  return EVERYTHING_OK;  
} 

